@@ -10,6 +10,7 @@ function core.register_chatcommand(cmd, def)
 	def.params = def.params or ""
 	def.description = def.description or ""
 	def.privs = def.privs or {}
+	def.mod_origin = core.get_current_modname() or "??"
 	core.chatcommands[cmd] = def
 end
 
@@ -37,6 +38,7 @@ core.register_on_chat_message(function(name, message)
 	end
 	local has_privs, missing_privs = core.check_player_privs(name, cmd_def.privs)
 	if has_privs then
+		core.set_last_run_mod(cmd_def.mod_origin)
 		local success, message = cmd_def.func(name, param)
 		if message then
 			core.chat_send_player(name, message)
@@ -49,6 +51,27 @@ core.register_on_chat_message(function(name, message)
 	return true  -- Handled chat message
 end)
 
+-- Parses a "range" string in the format of "here (number)" or
+-- "(x1, y1, z1) (x2, y2, z2)", returning two position vectors
+local function parse_range_str(player_name, str)
+	local p1, p2
+	local args = str:split(" ")
+
+	if args[1] == "here" then
+		p1, p2 = core.get_player_radius_area(player_name, tonumber(args[2]))
+		if p1 == nil then
+			return false, "Unable to get player " .. player_name .. " position"
+		end
+	else
+		p1, p2 = core.string_to_area(str)
+		if p1 == nil then
+			return false, "Incorrect area format. Expected: (x1,y1,z1) (x2,y2,z2)"
+		end
+	end
+
+	return p1, p2
+end
+
 --
 -- Chat commands
 --
@@ -61,6 +84,18 @@ core.register_chatcommand("me", {
 	end,
 })
 
+core.register_chatcommand("admin", {
+	description = "Show the name of the server owner",
+	func = function(name)
+		local admin = minetest.setting_get("name")
+		if admin then
+			return true, "The administrator of this server is "..admin.."."
+		else
+			return false, "There's no administrator named in the config file."
+		end
+	end,
+})
+
 core.register_chatcommand("help", {
 	privs = {},
 	params = "[all/privs/<cmd>]",
@@ -81,7 +116,7 @@ core.register_chatcommand("help", {
 			local cmds = {}
 			for cmd, def in pairs(core.chatcommands) do
 				if core.check_player_privs(name, def.privs) then
-					table.insert(cmds, cmd)
+					cmds[#cmds + 1] = cmd
 				end
 			end
 			table.sort(cmds)
@@ -92,7 +127,7 @@ core.register_chatcommand("help", {
 			local cmds = {}
 			for cmd, def in pairs(core.chatcommands) do
 				if core.check_player_privs(name, def.privs) then
-					table.insert(cmds, format_help_line(cmd, def))
+					cmds[#cmds + 1] = format_help_line(cmd, def)
 				end
 			end
 			table.sort(cmds)
@@ -100,7 +135,7 @@ core.register_chatcommand("help", {
 		elseif param == "privs" then
 			local privs = {}
 			for priv, def in pairs(core.registered_privileges) do
-				table.insert(privs, priv .. ": " .. def.description)
+				privs[#privs + 1] = priv .. ": " .. def.description
 			end
 			table.sort(privs)
 			return true, "Available privileges:\n"..table.concat(privs, "\n")
@@ -146,8 +181,10 @@ core.register_chatcommand("grant", {
 		end
 		local privs = core.get_player_privs(grantname)
 		local privs_unknown = ""
+		local basic_privs =
+			core.string_to_privs(core.setting_get("basic_privs") or "interact,shout")
 		for priv, _ in pairs(grantprivs) do
-			if priv ~= "interact" and priv ~= "shout" and
+			if not basic_privs[priv] and
 					not core.check_player_privs(name, {privs=true}) then
 				return false, "Your privileges are insufficient."
 			end
@@ -188,8 +225,10 @@ core.register_chatcommand("revoke", {
 		end
 		local revoke_privs = core.string_to_privs(revoke_priv_str)
 		local privs = core.get_player_privs(revoke_name)
+		local basic_privs =
+			core.string_to_privs(core.setting_get("basic_privs") or "interact,shout")
 		for priv, _ in pairs(revoke_privs) do
-			if priv ~= "interact" and priv ~= "shout" and priv ~= "interact_extra" and
+			if not basic_privs[priv] and
 					not core.check_player_privs(name, {privs=true}) then
 				return false, "Your privileges are insufficient."
 			end
@@ -229,21 +268,28 @@ core.register_chatcommand("setpassword", {
 		if not toname then
 			return false, "Name field required"
 		end
-		local actstr = "?"
+		local act_str_past = "?"
+		local act_str_pres = "?"
 		if not raw_password then
 			core.set_player_password(toname, "")
-			actstr = "cleared"
+			act_str_past = "cleared"
+			act_str_pres = "clears"
 		else
 			core.set_player_password(toname,
 					core.get_password_hash(toname,
 							raw_password))
-			actstr = "set"
+			act_str_past = "set"
+			act_str_pres = "sets"
 		end
 		if toname ~= name then
 			core.chat_send_player(toname, "Your password was "
-					.. actstr .. " by " .. name)
+					.. act_str_past .. " by " .. name)
 		end
-		return true, "Password of player \"" .. toname .. "\" " .. actstr
+
+		core.log("action", name .. " " .. act_str_pres
+		.. " password of " .. toname .. ".")
+
+		return true, "Password of player \"" .. toname .. "\" " .. act_str_past
 	end,
 })
 
@@ -257,6 +303,9 @@ core.register_chatcommand("clearpassword", {
 			return false, "Name field required"
 		end
 		core.set_player_password(toname, '')
+
+		core.log("action", name .. " clears password of " .. toname .. ".")
+
 		return true, "Password of player \"" .. toname .. "\" cleared"
 	end,
 })
@@ -303,12 +352,18 @@ core.register_chatcommand("teleport", {
 		p.x = tonumber(p.x)
 		p.y = tonumber(p.y)
 		p.z = tonumber(p.z)
-		teleportee = core.get_player_by_name(name)
-		if teleportee and p.x and p.y and p.z then
-			teleportee:setpos(p)
-			return true, "Teleporting to "..core.pos_to_string(p)
+		if p.x and p.y and p.z then
+			local lm = tonumber(minetest.setting_get("map_generation_limit") or 31000)
+			if p.x < -lm or p.x > lm or p.y < -lm or p.y > lm or p.z < -lm or p.z > lm then
+				return false, "Cannot teleport out of map bounds!"
+			end
+			teleportee = core.get_player_by_name(name)
+			if teleportee then
+				teleportee:setpos(p)
+				return true, "Teleporting to "..core.pos_to_string(p)
+			end
 		end
-		
+
 		local teleportee = nil
 		local p = nil
 		local target_name = nil
@@ -345,7 +400,7 @@ core.register_chatcommand("teleport", {
 			return true, "Teleporting " .. teleportee_name
 					.. " to " .. core.pos_to_string(p)
 		end
-		
+
 		local teleportee = nil
 		local p = nil
 		local teleportee_name = nil
@@ -367,7 +422,7 @@ core.register_chatcommand("teleport", {
 					.. " to " .. target_name
 					.. " at " .. core.pos_to_string(p)
 		end
-		
+
 		return false, 'Invalid parameters ("' .. param
 				.. '") or player not found (see /help teleport)'
 	end,
@@ -403,33 +458,65 @@ core.register_chatcommand("set", {
 	end,
 })
 
+local function emergeblocks_callback(pos, action, num_calls_remaining, ctx)
+	if ctx.total_blocks == 0 then
+		ctx.total_blocks   = num_calls_remaining + 1
+		ctx.current_blocks = 0
+	end
+	ctx.current_blocks = ctx.current_blocks + 1
+
+	if ctx.current_blocks == ctx.total_blocks then
+		core.chat_send_player(ctx.requestor_name,
+			string.format("Finished emerging %d blocks in %.2fms.",
+			ctx.total_blocks, (os.clock() - ctx.start_time) * 1000))
+	end
+end
+
+local function emergeblocks_progress_update(ctx)
+	if ctx.current_blocks ~= ctx.total_blocks then
+		core.chat_send_player(ctx.requestor_name,
+			string.format("emergeblocks update: %d/%d blocks emerged (%.1f%%)",
+			ctx.current_blocks, ctx.total_blocks,
+			(ctx.current_blocks / ctx.total_blocks) * 100))
+
+		core.after(2, emergeblocks_progress_update, ctx)
+	end
+end
+
+core.register_chatcommand("emergeblocks", {
+	params = "(here [radius]) | (<pos1> <pos2>)",
+	description = "starts loading (or generating, if inexistent) map blocks "
+		.. "contained in area pos1 to pos2",
+	privs = {server=true},
+	func = function(name, param)
+		local p1, p2 = parse_range_str(name, param)
+		if p1 == false then
+			return false, p2
+		end
+
+		local context = {
+			current_blocks = 0,
+			total_blocks   = 0,
+			start_time     = os.clock(),
+			requestor_name = name
+		}
+
+		core.emerge_area(p1, p2, emergeblocks_callback, context)
+		core.after(2, emergeblocks_progress_update, context)
+
+		return true, "Started emerge of area ranging from " ..
+			core.pos_to_string(p1, 1) .. " to " .. core.pos_to_string(p2, 1)
+	end,
+})
+
 core.register_chatcommand("deleteblocks", {
-	params = "[here] [<pos1> <pos2>]",
+	params = "(here [radius]) | (<pos1> <pos2>)",
 	description = "delete map blocks contained in area pos1 to pos2",
 	privs = {server=true},
 	func = function(name, param)
-		local p1 = {}
-		local p2 = {}
-		if param == "here" then
-			local player = core.get_player_by_name(name)
-			if player == nil then
-				core.log("error", "player is nil")
-				return false, "Unable to get current position; player is nil"
-			end
-			p1 = player:getpos()
-			p2 = p1
-		else
-			local pos1, pos2 = unpack(param:split(") ("))
-			if pos1 == nil or pos2 == nil then
-				return false, "Incorrect area format. Expected: (x1,y1,z1) (x2,y2,z2)"
-			end
-
-			p1 = core.string_to_pos(pos1 .. ")")
-			p2 = core.string_to_pos("(" .. pos2)
-
-			if p1 == nil or p2 == nil then
-				return false, "Incorrect area format. Expected: (x1,y1,z1) (x2,y2,z2)"
-			end
+		local p1, p2 = parse_range_str(name, param)
+		if p1 == false then
+			return false, p2
 		end
 
 		if core.delete_area(p1, p2) then
@@ -513,22 +600,29 @@ core.register_chatcommand("giveme", {
 })
 
 core.register_chatcommand("spawnentity", {
-	params = "<EntityName>",
-	description = "Spawn entity at your position",
+	params = "<EntityName> [<X>,<Y>,<Z>]",
+	description = "Spawn entity at given (or your) position",
 	privs = {give=true, interact=true},
 	func = function(name, param)
-		local entityname = string.match(param, "(.+)$")
+		local entityname, p = string.match(param, "^([^ ]+) *(.*)$")
 		if not entityname then
 			return false, "EntityName required"
 		end
-		core.log("action", ("/spawnentity invoked, entityname=%q")
-				:format(entityname))
+		core.log("action", ("%s invokes /spawnentity, entityname=%q")
+				:format(name, entityname))
 		local player = core.get_player_by_name(name)
 		if player == nil then
 			core.log("error", "Unable to spawn entity, player is nil")
 			return false, "Unable to spawn entity, player is nil"
 		end
-		local p = player:getpos()
+		if p == "" then
+			p = player:getpos()
+		else
+			p = core.string_to_pos(p)
+			if p == nil then
+				return false, "Invalid parameters ('" .. param .. "')"
+			end
+		end
 		p.y = p.y + 1
 		core.add_entity(p, entityname)
 		return true, ("%q spawned."):format(entityname)
@@ -662,23 +756,52 @@ core.register_chatcommand("status", {
 })
 
 core.register_chatcommand("time", {
-	params = "<0...24000>",
+	params = "<0..23>:<0..59> | <0..24000>",
 	description = "set time of day",
-	privs = {settime=true},
+	privs = {},
 	func = function(name, param)
 		if param == "" then
-			return false, "Missing time."
+			local current_time = math.floor(core.get_timeofday() * 1440)
+			local minutes = current_time % 60
+			local hour = (current_time - minutes) / 60
+			return true, ("Current time is %d:%02d"):format(hour, minutes)
 		end
-		local newtime = tonumber(param)
-		if newtime == nil then
-			return false, "Invalid time."
+		local player_privs = core.get_player_privs(name)
+		if not player_privs.settime then
+			return false, "You don't have permission to run this command " ..
+				"(missing privilege: settime)."
 		end
-		core.set_timeofday((newtime % 24000) / 24000)
-		core.log("action", name .. " sets time " .. newtime)
+		local hour, minute = param:match("^(%d+):(%d+)$")
+		if not hour then
+			local new_time = tonumber(param)
+			if not new_time then
+				return false, "Invalid time."
+			end
+			-- Backward compatibility.
+			core.set_timeofday((new_time % 24000) / 24000)
+			core.log("action", name .. " sets time to " .. new_time)
+			return true, "Time of day changed."
+		end
+		hour = tonumber(hour)
+		minute = tonumber(minute)
+		if hour < 0 or hour > 23 then
+			return false, "Invalid hour (must be between 0 and 23 inclusive)."
+		elseif minute < 0 or minute > 59 then
+			return false, "Invalid minute (must be between 0 and 59 inclusive)."
+		end
+		core.set_timeofday((hour * 60 + minute) / 1440)
+		core.log("action", ("%s sets time to %d:%02d"):format(name, hour, minute))
 		return true, "Time of day changed."
 	end,
 })
 
+core.register_chatcommand("days", {
+	description = "Display day count",
+	func = function(name, param)
+		return true, "Current day is " .. core.get_day_count()
+	end
+})
+
 core.register_chatcommand("shutdown", {
 	description = "shutdown server",
 	privs = {server=true},
@@ -732,20 +855,35 @@ core.register_chatcommand("kick", {
 		if not core.kick_player(tokick, reason) then
 			return false, "Failed to kick player " .. tokick
 		end
-		core.log("action", name .. " kicked " .. tokick)
+		local log_reason = ""
+		if reason then
+			log_reason = " with reason \"" .. reason .. "\""
+		end
+		core.log("action", name .. " kicks " .. tokick .. log_reason)
 		return true, "Kicked " .. tokick
 	end,
 })
 
 core.register_chatcommand("clearobjects", {
+	params = "[full|quick]",
 	description = "clear all objects in world",
 	privs = {server=true},
 	func = function(name, param)
-		core.log("action", name .. " clears all objects.")
+		local options = {}
+		if param == "" or param == "full" then
+			options.mode = "full"
+		elseif param == "quick" then
+			options.mode = "quick"
+		else
+			return false, "Invalid usage, see /help clearobjects."
+		end
+
+		core.log("action", name .. " clears all objects ("
+				.. options.mode .. " mode).")
 		core.chat_send_all("Clearing all objects.  This may take long."
 				.. "  You may experience a timeout.  (by "
 				.. name .. ")")
-		core.clear_objects()
+		core.clear_objects(options)
 		core.log("action", "Object clearing done.")
 		core.chat_send_all("*** Cleared all objects.")
 	end,
@@ -788,4 +926,3 @@ core.register_chatcommand("last-login", {
 		return false, "Last login time is unknown"
 	end,
 })
-
